﻿using System;
using System.Threading;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Utilities;

namespace PeterHeibrink.Example.SelfServiceSiteCreation.WebParts
{
    /// <summary>
    /// This <see cref="WebPart"/> illustrates the use of <see cref="SPSite.SelfServiceCreateSite"/>.
    /// </summary>
    /// <remarks>
    /// This <see cref="WebPart"/> is meant as an example. For real life application more work has to be done
    /// for example for validation. Important aspect could be:
    /// <list type="bullet">
    /// <item>Is there already a <see cref="SPSite"/> with the provided relative url</item>
    /// <item>Are the characters used allowed? F.e. weird characters during <see cref="SPSite.SelfServiceCreateSite"/> will cause exceptions.</item>
    /// <item>Are all users allowed to use the <see cref="WebPart"/> or only contributors</item>
    /// <item>The administrator can set the secundary site collection administrator to mandatory, you then have to do some validating as well.</item>
    /// <item>Etc...</item>
    /// </list>
    /// </remarks>
    public class SelfServiceSiteCreationWebPart : WebPart
    {
        #region Global variables

        DropDownList _uxManagedPath;
        TextBox _uxRelativeSiteUrl;
        TextBox _uxTitle;
        TextBox _uxDescription;
        DropDownList _uxWebTemplate;
        TextBox _uxSecondarySiteAdminLogin;
        TextBox _uxSecondarySiteAdminName;
        TextBox _uxSecondarySiteAdminEmail;

        #endregion

        #region Creation of the controls displayed on the WebPart

        /// <summary>
        /// This function creates the <see cref="Table"/> which allows the user to enter the
        /// information for the site which needs to be generated
        /// </summary>
        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            // If self service site creation is not enabled, only an error message is shown
            if (!SPContext.Current.Site.WebApplication.SelfServiceSiteCreationEnabled)
            {
                Controls.Add(CreateErrorLabel("Self-Service Site Management is not enabled. Please go to \"Central Administration > Application Management > Self-service Site Management\" and enable it for this Web Application before continuing."));
                return;
            }

            // Assume the worst, no managed paths are available, and so SelfServiceCreateSite cannot work
            if (!IsManagedPathAvailable())
            {
                Controls.Add(CreateErrorLabel("No managed paths are available. Please go to \"Central Administration > Application Management > Define Managed Paths\" and define at least one managed path before continuing."));
                return;
            }

            // Create table to style the properties
            Table containerTable = new Table();
            containerTable.Style.Add("vertical-align", "top");

            // Add the header row
            TableHeaderRow headerRow = new TableHeaderRow();
            TableHeaderCell headerFieldCell = new TableHeaderCell();
            headerFieldCell.Style.Add("width", "250px");
            headerFieldCell.Controls.Add(new LiteralControl("Field"));
            TableHeaderCell headervalueCell = new TableHeaderCell();
            headervalueCell.Controls.Add(new LiteralControl("Value"));
            headerRow.Controls.Add(headerFieldCell);
            headerRow.Controls.Add(headervalueCell);
            headerRow.Style.Add("text-align", "left");
            containerTable.Controls.Add(headerRow);

            // Add the row for the relative site url
            _uxRelativeSiteUrl = new TextBox();
            _uxManagedPath = new DropDownList();
            _uxManagedPath.DataSource = SPContext.Current.Site.WebApplication.Prefixes;
            _uxManagedPath.DataTextField = "Name";
            _uxManagedPath.DataValueField = "Name";
            _uxManagedPath.DataBind();
            // Remove the (root) managed path since we can't create a second site there
            _uxManagedPath.Items.Remove(new ListItem(string.Empty, string.Empty));
            Panel relativeSiteUrlControls = new Panel();
            relativeSiteUrlControls.Controls.Add(_uxManagedPath);
            relativeSiteUrlControls.Controls.Add(_uxRelativeSiteUrl);
            containerTable.Controls.Add(CreateTableRow("Relative Site Url", relativeSiteUrlControls));
           
            // Add the row for the title
            _uxTitle = new TextBox();
            containerTable.Controls.Add(CreateTableRow("Title", _uxTitle));

            // Add the row for the description
            _uxDescription = new TextBox();
            _uxDescription.TextMode = TextBoxMode.MultiLine;
            containerTable.Controls.Add(CreateTableRow("Description", _uxDescription));

            // Add the row for the web template
            _uxWebTemplate = new DropDownList();
            _uxWebTemplate.DataSource = SPContext.Current.Site.GetWebTemplates((uint)SPContext.Current.Web.Locale.LCID);
            _uxWebTemplate.DataTextField = "Title";
            _uxWebTemplate.DataValueField = "Name";
            _uxWebTemplate.DataBind();
            containerTable.Controls.Add(CreateTableRow("Web Template", _uxWebTemplate));

            // Add the row for the secondary site collection login
            _uxSecondarySiteAdminLogin = new TextBox();
            containerTable.Controls.Add(CreateTableRow("Secondary site admin login", _uxSecondarySiteAdminLogin));

            // Add the row for the secondary site collection name
            _uxSecondarySiteAdminName = new TextBox();
            containerTable.Controls.Add(CreateTableRow("Secondary site admin name", _uxSecondarySiteAdminName));

            // Add the row for the secondary site collection name
            _uxSecondarySiteAdminEmail = new TextBox();
            containerTable.Controls.Add(CreateTableRow("Secondary site admin email", _uxSecondarySiteAdminEmail));

            // Add the final row with the submit button
            Button uxSubmitButton = new Button();
            uxSubmitButton.Text = "Submit";
            uxSubmitButton.Click += new EventHandler(SubmitButton_Click);
            containerTable.Controls.Add(CreateTableRow(string.Empty, uxSubmitButton));

            // Add the table to the controls of the WebPart
            Controls.Add(containerTable);
        }

        #endregion

        #region SelfServiceCreateSite Logic

        /// <summary>
        /// This function will create a site collection using <see cref="SPSite.SelfServiceCreateSite"/>
        /// </summary>
        /// <param name="relativeSiteUrl">
        /// The relative url of a site. This should include a managed path since the root site 
        /// collection already exist. Example "sites/testsite"
        /// </param>
        /// <param name="title">The title of the new site</param>
        /// <param name="description">The description of the new site</param>
        /// <param name="webTemplate">The web template name</param>
        /// <param name="secondarySiteCollectionAdministratorLogin">
        /// The user login account of the secondary site collection administrator
        /// </param>
        /// <param name="secondarySiteCollectionAdministratorName">
        /// The name of the secondary site collection administrator
        /// </param>
        /// <param name="secondarySiteCollectionAdministratorEmail">
        /// The email address of the secondary site collection administrator
        /// </param>
        /// <returns>
        /// The newly create <see cref="SPSite"/> or null when an error has occured.
        /// </returns>
        private SPSite CreateNewSiteCollection(string relativeSiteUrl,
                                                string title,
                                                string description,
                                                string webTemplate,
                                                string secondarySiteCollectionAdministratorLogin,
                                                string secondarySiteCollectionAdministratorName,
                                                string secondarySiteCollectionAdministratorEmail)
        {
            SPSite newSiteCollection = null;

            // The creation will only work if you set the primary administrator is set to the current user
            string primarySiteCollectionAdministratorLogin = SPContext.Current.Web.CurrentUser.LoginName;
            string primarySiteCollectionAdministratorName = SPContext.Current.Web.CurrentUser.Name;
            string primarySiteCollectionAdministratorEmail = SPContext.Current.Web.CurrentUser.Email;

            // Small change you want to set this variable on a different value as the root site collection
            uint localIdentifier = (uint)SPContext.Current.Web.Locale.LCID;

            try
            {
                // Make sure we are at the root site collection
                using (SPSite rootSite = new SPSite(GetAuthorityUrl(SPContext.Current.Web.Url)))
                {
                    // If no secondary site collection administrator is provided, use null values
                    // in stead of an empty string. Otherwise an Exception will be thrown.
                    if (string.IsNullOrEmpty(secondarySiteCollectionAdministratorLogin))
                        newSiteCollection = rootSite.SelfServiceCreateSite(
                                                relativeSiteUrl,
                                                title,
                                                description,
                                                localIdentifier,
                                                webTemplate,
                                                primarySiteCollectionAdministratorLogin,
                                                primarySiteCollectionAdministratorName,
                                                primarySiteCollectionAdministratorEmail,
                                                null,
                                                null,
                                                null);
                    else
                    {
                        // When a secundary site collection has been provided, add this account as a
                        // secundary site collection administrator
                        newSiteCollection = rootSite.SelfServiceCreateSite(
                                                relativeSiteUrl,
                                                title,
                                                description,
                                                localIdentifier,
                                                webTemplate,
                                                primarySiteCollectionAdministratorLogin,
                                                primarySiteCollectionAdministratorName,
                                                primarySiteCollectionAdministratorEmail,
                                                secondarySiteCollectionAdministratorLogin,
                                                secondarySiteCollectionAdministratorName,
                                                secondarySiteCollectionAdministratorEmail);
                    }
                }
            }
            catch
            {
                // You can implement some logging here. It can for example happen that you provide 
                // a URL of a site that already exist, a template that not exist etc.
            }

            return newSiteCollection;
        }

        #endregion

        #region Event handler

        /// <summary>
        /// This event handles the submit button within the <see cref="WebPart"/>. It will start the 
        /// creation of the new site and uses the <see cref="SPLongOperation"/> to show the user a
        /// progress screen
        /// </summary>
        /// <param name="sender">The sender object (<see cref="Button"/>)</param>
        /// <param name="e">The event arguments</param>
        private void SubmitButton_Click(object sender, EventArgs e)
        {
            // Get the information from the user controls
            string relativeSiteUrl = string.Format("{0}/{1}",_uxManagedPath.SelectedValue,_uxRelativeSiteUrl.Text);
            string title = _uxTitle.Text;
            string description = _uxDescription.Text;
            string webTemplate = _uxWebTemplate.SelectedValue;
            string secundarySiteCollectionAdministratorLogin = _uxSecondarySiteAdminLogin.Text;
            string secundarySiteCollectionAdministratorName = _uxSecondarySiteAdminName.Text;
            string secundarySiteCollectionAdministratorEmail = _uxSecondarySiteAdminEmail.Text;

            // Create a new SPLongOperation
            SPLongOperation longOperation = new SPLongOperation(this.Page);

            // Provide the text displayd in bold
            longOperation.LeadingHTML = "Creating site collection";

            // Provide the normal formatted text
            longOperation.TrailingHTML = "Please wait while your request is being performed. You will be redirected to the newly created site collection";

            // Let's start the code that takes a while from here
            longOperation.Begin();
            try
            {
                string successUrl = SPContext.Current.Web.Url;

                // Create the site collection
                using (SPSite newSite = CreateNewSiteCollection(relativeSiteUrl,
                    title,
                    description,
                    webTemplate,
                    secundarySiteCollectionAdministratorLogin,
                    secundarySiteCollectionAdministratorName,
                    secundarySiteCollectionAdministratorEmail))
                {
                    if (newSite != null)
                        successUrl = newSite.RootWeb.Url;
                }

                /// When the action is performed, the page will be redirect to this url
                longOperation.End(successUrl);
            }
            catch (ThreadAbortException)
            {
                // Don't do anything, this error can occur because the SPLongOperation.End
                // performs a Response.Redirect internally and doesnt take into account that other code
                // might still be executed
            }
            catch (Exception ex)
            {
                // When an exception occurs, the page is redirected to the error page.
                // Redirection to another (custom) page is also possible
                SPUtility.TransferToErrorPage(ex.ToString());
            }
        }

        #endregion

        #region Helper functions

        /// <summary>
        /// This function checks if the current <see cref="SPWebApplication"/> contains managed
        /// paths. Otherwise <see cref="SPSite.SelfServiceCreateSite"/> will not work.
        /// </summary>
        /// <returns><see cref="bool"/> indicating if managed paths are available</returns>
        private static bool IsManagedPathAvailable()
        {
            bool isManagedPathAvailable = false;
            foreach (SPPrefix prefix in SPContext.Current.Site.WebApplication.Prefixes)
            {
                if (!prefix.Name.Equals(string.Empty))
                {
                    isManagedPathAvailable = true;
                    break;
                }
            }
            return isManagedPathAvailable;
        }

        /// <summary>
        /// Returns a URL with only the scheme and authority segmenst from a full Url.
        /// This will generate the url of the root site collection
        /// </summary>
        /// <param name="url">The full URL to format.</param>
        /// <returns>Returns the URL without the path and querystring.</returns>
        /// <example>http://devserver</example>
        private string GetAuthorityUrl(string url)
        {
            Uri uri = new Uri(url);
            return string.Format(@"{0}://{1}", uri.Scheme, uri.Authority);
        }

        /// <summary>
        /// This function creates a new <see cref="TableRow"/> for a given title and
        /// control
        /// </summary>
        /// <param name="title">Title to display</param>
        /// <param name="valueControl">Control to add</param>
        /// <returns>New TableRow</returns>
        private TableRow CreateTableRow(string title, Control valueControl)
        {
            TableRow newRow = new TableRow();

            TableCell newFieldCell = new TableCell();
            newFieldCell.Controls.Add(new LiteralControl(title));

            TableCell newlValueCell = new TableCell();
            newlValueCell.Controls.Add(valueControl);
            newRow.Controls.Add(newFieldCell);
            newRow.Controls.Add(newlValueCell);
            return newRow;
        }

        /// <summary>
        /// This function creates a new <see cref="Label"/> which displays an errormessage
        /// </summary>
        /// <param name="errorMessage">Error message to display</param>
        /// <returns>New Label</returns>
        private Label CreateErrorLabel(string errorMessage)
        {
            Label errorMessageLabel = new Label();
            errorMessageLabel.Text = errorMessage;
            errorMessageLabel.Style.Add("color", "Red");
            errorMessageLabel.Style.Add("font-weight", "bold");
            return errorMessageLabel;
        }

        #endregion
    }
}